/**************************************************************
RC5 decoder programma - Bart Huyskens - SJI Schoten - 16/06/2013

Sensor: TSOP1236 - aangesloten aan PIN RB0 (Externe interrupt)
Deze Sensor levert een geinverteerd signaal aan pin RB0
Connecteer: 
TSOP1236 aan RB0 
leds aan POORTD
LCD (B7-A3)(B6-A2)(B5-A1)(B4-A0)(EN-A4)(RS-A5)

Principe: 
- Via externe int aan RB0 wordt de eerste (dalende) flank gedetecteerd
- 1.331msec later = 889 + (889/2) wordt gekeken of het signaal hoog of laag is. hoog = 0, laag = 1
- die 1.331 msec wordt gemeten via het TMR1H register van TMR1 - DIt TMRH register staat op 25 als 1.331msec gepasseerd is.
- dit wordt herhaald voor alle 14 bits - deze 14 bits schuiven n voor n in het RC5CODE_16BIT register.
- onvangen bericht wordt zichtbaar gemaakt op de LCD en op de leds aan poort D

***************************************************************/

#include <htc.h>
#include "HITECH_LCD.h"		// include LCD library

#define _XTAL_FREQ  19660800						 
#define TMR1H_FREQ (_XTAL_FREQ/(4*1*256)) 			//Xtal freq / 4 / prescaler 1:1 / TMR1L (256) = 19200Hz = 52.08usec
#define BITFREQ  751								// 75% of bittime = 889 + (889/2) = 1,331msec = 751,31Hz
#define RELBIT (TMR1H_FREQ/BITFREQ)					// 25 - 1,331msec / 52.08usec = 25 - TMR1H zal op 25 staan na 1.331msec
										
unsigned char ADRES , DATA = 0;
unsigned char BITCOUNTER = 0;
unsigned int RC5CODE_16BIT, RC5CODE_DONE = 0;

__CONFIG(0x23F2);   		//extern Xtal HS 19660800
__CONFIG(0x3FFF);			//

void main()				
{
TRISD = 0x00; 				// leds aan poortD = output
PORTD = 0x00;
ANSEL = 0x00;
ANSELH = 0X00;
TRISB = 0x01;				//RB0 externe interrupt moet input zijn
LCD_Start();				// Initialiseer LCD

// TMR1 settings
TMR1IE=1;					// Enable TMR1 overflow interrupt - geeft aan dat er 13msec gepasseerd zijn

// External Interrupt setting - RB0
INTEDG=0; 					//falling edge - vermits de TSOP sensor een inver signaal geeft - zoeken we de eerste dalende flank.
INTE = 1;					// Enable externe interrupt
PEIE = 1;					// Enable peripheral interrupt
GIE = 1;					// Enable global interrupt

	while (1)
		{
		DATA = RC5CODE_DONE & 0b0000000000111111; 				// 6 LSB's zijn de data
		ADRES = (RC5CODE_DONE>>6)&0b0000000000011111;			// Bit 6-10 zijn het adres
		
		LCD_Clear();
		LCD_PrintString("RC5 DECODER");
		LCD_Cursor(0, 1);
		LCD_PrintString("ADR:");		
		LCD_PrintNumber(ADRES);		
		LCD_Cursor(8, 1);
		LCD_PrintString("DAT:");			
		LCD_PrintNumber(DATA);	
		__delay_ms(50);		// flikkering LCD wegwerken
		}

}   // einde void main ftie


void interrupt isr(void)  // interrupt - gevoelig voor TMR1 overflow en flank op RB0
{
if(TMR1IF) 								//check the timer overflow - elke +/-13msec
	{
		TMR1IF=0; 						// reset de interrupt flag
		TMR1ON=0; 						// stop de timer
		INTEDG=0;						// een nieuw RC5 signaal begint steeds met een dalende flank
		RC5CODE_DONE = RC5CODE_16BIT;	// Als er 13msec geen flank gedetecteerd is, dan nemen we aan dat de data binnen is 
										// Deze data wordt gecopieerd naar het 16 bit RC5CODE_DONE register
	
		PORTD =RC5CODE_DONE & 0b00111111;			// Zet 6 lsb's (data)op de leds
		
	}
else if (INTF) 						// controleer of er een flank is op RB0
	{
		INTF=0;						// reset the interrupt
		INTEDG = !INTEDG; 			// verander de flankgevoeligheid van hoog naar laag of omgekeerd

		if(!TMR1ON)					// ALs de timer nog niet aan staat, dan betekent dit dat de eerste bit van het RC5 signaal wordt binnengehaald
		{
			BITCOUNTER=13;			// zet de bitcounter op 13 om alle 14 bits binnen te halen
			TMR1H=0;				// reset the timer
			TMR1L=0;				// 
			TMR1ON=1;				// start the timer
		}
	
		else						// doe dit als het niet de eerste bit is die je binnen haalt
		{ 							
		if(TMR1H>RELBIT)				// controleer of er reeds 1331 uS gepasseerd zijn - als dit zo is, haal dan deze nieuwe bitwaarde binnen
			{					
				RC5CODE_16BIT = RC5CODE_16BIT <<1;			// Schuif alle bits van de 16 bit RC5CODE_16BIT n plaats naar links
				if(!RB0)									// als RB0 op dit moment 0 is, dan moet er een 1 in het RC5CODE_16BIT register geladen worden
					{
					RC5CODE_16BIT = RC5CODE_16BIT | 0b0000000000000001;		// 	// laad een 1 in de LSB van het RC5CODE_16BIT gerister
					}
				BITCOUNTER--;		// decrement de BITCOUNTER
				TMR1ON=0;			// off timer
				TMR1H=0;			// reset the timer
				TMR1L=0;			// 
				TMR1ON=1;			// start the timer
				PORTD = 0;			// alle leds uit
			}					
		}
	}


}


